#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: my_device_portal
short_description: Resource module for My Device Portal
description:
- Manage operations create, update and delete of the resource My Device Portal.
- This API creates a my device portal.
- This API deletes a my device portal by ID.
- This API allows the client to update a my device portal by ID.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  customizations:
    description: Defines all of the Portal Customizations available.
    suboptions:
      globalCustomizations:
        description: My Device Portal's globalCustomizations.
        suboptions:
          backgroundImage:
            description: My Device Portal's backgroundImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          bannerImage:
            description: My Device Portal's bannerImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          bannerTitle:
            description: My Device Portal's bannerTitle.
            type: str
          contactText:
            description: My Device Portal's contactText.
            type: str
          desktopLogoImage:
            description: My Device Portal's desktopLogoImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          footerElement:
            description: My Device Portal's footerElement.
            type: str
          mobileLogoImage:
            description: My Device Portal's mobileLogoImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
        type: dict
      language:
        description: This property is supported only for Read operation and it allows
          to show the customizations in English. Other languages are not supported.
        suboptions:
          viewLanguage:
            description: My Device Portal's viewLanguage.
            type: str
        type: dict
      pageCustomizations:
        description: Represent the entire page customization as a giant dictionary.
        suboptions:
          data:
            description: The Dictionary will be exposed here as key value pair.
            elements: dict
            suboptions:
              key:
                description: My Device Portal's key.
                type: str
              value:
                description: My Device Portal's value.
                type: str
            type: list
        type: dict
      portalTheme:
        description: My Device Portal's portalTheme.
        suboptions:
          id:
            description: My Device Portal's id.
            type: str
          name:
            description: The system- or user-assigned name of the portal theme.
            type: str
          themeData:
            description: A CSS file, represented as a Base64-encoded byte array.
            type: str
        type: dict
      portalTweakSettings:
        description: The Tweak Settings are a customization of the Portal Theme that
          has been selected for the portal. When the Portal Theme selection is changed,
          the Tweak Settings are overwritten to match the values in the theme. The Tweak
          Settings can subsequently be changed by the user.
        suboptions:
          bannerColor:
            description: Hex value of color.
            type: str
          bannerTextColor:
            description: My Device Portal's bannerTextColor.
            type: str
          pageBackgroundColor:
            description: My Device Portal's pageBackgroundColor.
            type: str
          pageLabelAndTextColor:
            description: My Device Portal's pageLabelAndTextColor.
            type: str
        type: dict
    type: dict
  description:
    description: My Device Portal's description.
    type: str
  id:
    description: My Device Portal's id.
    type: str
  name:
    description: My Device Portal's name.
    type: str
  portalTestUrl:
    description: URL to bring up a test page for this portal.
    type: str
  portalType:
    description: Allowed values - BYOD, - HOTSPOTGUEST, - MYDEVICE, - SELFREGGUEST,
      - SPONSOR, - SPONSOREDGUEST.
    type: str
  settings:
    description: Defines all of the settings groups available for a Mydevice portal.
    suboptions:
      aupSettings:
        description: Configuration of the Acceptable Use Policy (AUP) for a portal.
        suboptions:
          displayFrequency:
            description: How the AUP should be displayed, either on page or as a link.
              Only valid if includeAup = true. Allowed Values - FIRSTLOGIN, - EVERYLOGIN,
              - RECURRING.
            type: str
          displayFrequencyIntervalDays:
            description: Number of days between AUP confirmations (when displayFrequency
              = recurring).
            type: int
          includeAup:
            description: Require the portal user to read and accept an AUP.
            type: bool
          requireScrolling:
            description: Require the portal user to scroll to the end of the AUP. Only
              valid if requireAupAcceptance = true.
            type: bool
        type: dict
      employeeChangePasswordSettings:
        description: My Device Portal's employeeChangePasswordSettings.
        suboptions:
          allowEmployeeToChangePwd:
            description: AllowEmployeeToChangePwd flag.
            type: bool
        type: dict
      loginPageSettings:
        description: My Device Portal's loginPageSettings.
        suboptions:
          aupDisplay:
            description: How the AUP should be displayed, either on page or as a link.
              Only valid if includeAup = true. Allowed values - ONPAGE, - ASLINK.
            type: str
          includeAup:
            description: Include an Acceptable Use Policy (AUP) that should be displayed
              during login.
            type: bool
          maxFailedAttemptsBeforeRateLimit:
            description: Maximum failed login attempts before rate limiting.
            type: int
          requireAupAcceptance:
            description: Require the portal user to accept the AUP. Only valid if includeAup
              = true.
            type: bool
          requireScrolling:
            description: Require the portal user to scroll to the end of the AUP. Only
              valid if requireAupAcceptance = true.
            type: bool
          socialConfigs:
            description: My Device Portal's socialConfigs.
            elements: dict
            type: list
          timeBetweenLoginsDuringRateLimit:
            description: Time between login attempts when rate limiting.
            type: int
        type: dict
      portalSettings:
        description: The port, interface, certificate, and other basic settings of a
          portal.
        suboptions:
          allowedInterfaces:
            description: Interfaces that the portal will be reachable on. Allowed values
              - eth0, - eth1, - eth2, - eth3, - eth4, - eth5, - bond0, - bond1, - bond2.
            elements: str
            type: list
          alwaysUsedLanguage:
            description: My Device Portal's alwaysUsedLanguage.
            type: str
          certificateGroupTag:
            description: Logical name of the x.509 server certificate that will be used
              for the portal.
            type: str
          displayLang:
            description: Allowed values - USEBROWSERLOCALE, - ALWAYSUSE.
            type: str
          endpointIdentityGroup:
            description: Unique Id of the endpoint identity group where user's devices
              will be added. Used only in Hotspot Portal.
            type: str
          fallbackLanguage:
            description: Used when displayLang = USEBROWSERLOCALE.
            type: str
          httpsPort:
            description: The port number that the allowed interfaces will listen on.
              Range from 8000 to 8999.
            type: int
        type: dict
      postAccessBannerSettings:
        description: My Device Portal's postAccessBannerSettings.
        suboptions:
          includePostAccessBanner:
            description: IncludePostAccessBanner flag.
            type: bool
        type: dict
      postLoginBannerSettings:
        description: My Device Portal's postLoginBannerSettings.
        suboptions:
          includePostAccessBanner:
            description: Include a Post-Login Banner page.
            type: bool
        type: dict
      supportInfoSettings:
        description: My Device Portal's supportInfoSettings.
        suboptions:
          defaultEmptyFieldValue:
            description: The default value displayed for an empty field. Only valid
              when emptyFieldDisplay = DISPLAYWITHDEFAULTVALUE.
            type: str
          emptyFieldDisplay:
            description: Specifies how empty fields are handled on the Support Information
              Page. Allowed values - HIDE, - DISPLAYWITHNOVALUE, - DISPLAYWITHDEFAULTVALUE.
            type: str
          includeBrowserUserAgent:
            description: IncludeBrowserUserAgent flag.
            type: bool
          includeFailureCode:
            description: IncludeFailureCode flag.
            type: bool
          includeIpAddress:
            description: IncludeIpAddress flag.
            type: bool
          includeMacAddr:
            description: IncludeMacAddr flag.
            type: bool
          includePolicyServer:
            description: IncludePolicyServer flag.
            type: bool
          includeSupportInfoPage:
            description: IncludeSupportInfoPage flag.
            type: bool
        type: dict
    type: dict
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    my_device_portal.MyDevicePortal.create_my_device_portal,
    my_device_portal.MyDevicePortal.delete_my_device_portal_by_id,
    my_device_portal.MyDevicePortal.update_my_device_portal_by_id,

  - Paths used are
    post /ers/config/mydeviceportal,
    delete /ers/config/mydeviceportal/{id},
    put /ers/config/mydeviceportal/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.my_device_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customizations:
      globalCustomizations:
        backgroundImage:
          data: string
        bannerImage:
          data: string
        bannerTitle: string
        contactText: string
        desktopLogoImage:
          data: string
        footerElement: string
        mobileLogoImage:
          data: string
      language:
        viewLanguage: string
      pageCustomizations:
        data:
        - key: string
          value: string
      portalTheme:
        id: string
        name: string
        themeData: string
      portalTweakSettings:
        bannerColor: string
        bannerTextColor: string
        pageBackgroundColor: string
        pageLabelAndTextColor: string
    description: string
    id: string
    name: string
    portalTestUrl: string
    portalType: string
    settings:
      aupSettings:
        displayFrequency: string
        displayFrequencyIntervalDays: 0
        includeAup: true
        requireScrolling: true
      employeeChangePasswordSettings:
        allowEmployeeToChangePwd: true
      loginPageSettings:
        aupDisplay: string
        includeAup: true
        maxFailedAttemptsBeforeRateLimit: 0
        requireAupAcceptance: true
        requireScrolling: true
        socialConfigs:
        - {}
        timeBetweenLoginsDuringRateLimit: 0
      portalSettings:
        allowedInterfaces:
        - string
        alwaysUsedLanguage: string
        certificateGroupTag: string
        displayLang: string
        endpointIdentityGroup: string
        fallbackLanguage: string
        httpsPort: 0
      postAccessBannerSettings:
        includePostAccessBanner: true
      postLoginBannerSettings:
        includePostAccessBanner: true
      supportInfoSettings:
        defaultEmptyFieldValue: string
        emptyFieldDisplay: string
        includeBrowserUserAgent: true
        includeFailureCode: true
        includeIpAddress: true
        includeMacAddr: true
        includePolicyServer: true
        includeSupportInfoPage: true

- name: Delete by id
  cisco.ise.my_device_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.my_device_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customizations:
      globalCustomizations:
        backgroundImage:
          data: string
        bannerImage:
          data: string
        bannerTitle: string
        contactText: string
        desktopLogoImage:
          data: string
        footerElement: string
        mobileLogoImage:
          data: string
      language:
        viewLanguage: string
      pageCustomizations:
        data:
        - key: string
          value: string
      portalTheme:
        id: string
        name: string
        themeData: string
      portalTweakSettings:
        bannerColor: string
        bannerTextColor: string
        pageBackgroundColor: string
        pageLabelAndTextColor: string
    description: string
    name: string
    portalTestUrl: string
    portalType: string
    settings:
      aupSettings:
        displayFrequency: string
        displayFrequencyIntervalDays: 0
        includeAup: true
        requireScrolling: true
      employeeChangePasswordSettings:
        allowEmployeeToChangePwd: true
      loginPageSettings:
        aupDisplay: string
        includeAup: true
        maxFailedAttemptsBeforeRateLimit: 0
        requireAupAcceptance: true
        requireScrolling: true
        socialConfigs:
        - {}
        timeBetweenLoginsDuringRateLimit: 0
      portalSettings:
        allowedInterfaces:
        - string
        alwaysUsedLanguage: string
        certificateGroupTag: string
        displayLang: string
        endpointIdentityGroup: string
        fallbackLanguage: string
        httpsPort: 0
      postAccessBannerSettings:
        includePostAccessBanner: true
      postLoginBannerSettings:
        includePostAccessBanner: true
      supportInfoSettings:
        defaultEmptyFieldValue: string
        emptyFieldDisplay: string
        includeBrowserUserAgent: true
        includeFailureCode: true
        includeIpAddress: true
        includeMacAddr: true
        includePolicyServer: true
        includeSupportInfoPage: true

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "portalType": "string",
      "portalTestUrl": "string",
      "settings": {
        "portalSettings": {
          "httpsPort": 0,
          "allowedInterfaces": [
            "string"
          ],
          "certificateGroupTag": "string",
          "endpointIdentityGroup": "string",
          "displayLang": "string",
          "fallbackLanguage": "string",
          "alwaysUsedLanguage": "string"
        },
        "loginPageSettings": {
          "maxFailedAttemptsBeforeRateLimit": 0,
          "timeBetweenLoginsDuringRateLimit": 0,
          "includeAup": true,
          "aupDisplay": "string",
          "requireAupAcceptance": true,
          "requireScrolling": true,
          "socialConfigs": [
            {}
          ]
        },
        "aupSettings": {
          "displayFrequencyIntervalDays": 0,
          "displayFrequency": "string",
          "includeAup": true,
          "requireScrolling": true
        },
        "employeeChangePasswordSettings": {
          "allowEmployeeToChangePwd": true
        },
        "postLoginBannerSettings": {
          "includePostAccessBanner": true
        },
        "postAccessBannerSettings": {
          "includePostAccessBanner": true
        },
        "supportInfoSettings": {
          "includeSupportInfoPage": true,
          "includeMacAddr": true,
          "includeIpAddress": true,
          "includeBrowserUserAgent": true,
          "includePolicyServer": true,
          "includeFailureCode": true,
          "emptyFieldDisplay": "string",
          "defaultEmptyFieldValue": "string"
        }
      },
      "customizations": {
        "portalTheme": {
          "id": "string",
          "name": "string",
          "themeData": "string"
        },
        "portalTweakSettings": {
          "bannerColor": "string",
          "bannerTextColor": "string",
          "pageBackgroundColor": "string",
          "pageLabelAndTextColor": "string"
        },
        "language": {
          "viewLanguage": "string"
        },
        "globalCustomizations": {
          "mobileLogoImage": {
            "data": "string"
          },
          "desktopLogoImage": {
            "data": "string"
          },
          "bannerImage": {
            "data": "string"
          },
          "backgroundImage": {
            "data": "string"
          },
          "bannerTitle": "string",
          "contactText": "string",
          "footerElement": "string"
        },
        "pageCustomizations": {
          "data": [
            {
              "key": "string",
              "value": "string"
            }
          ]
        }
      },
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2024 Aubin Bikouo (@abikouo)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: rds_engine_versions_info
version_added: 7.6.0
short_description: Describes the properties of specific versions of DB engines
description:
  - Obtain information about a specific versions of DB engines.
options:
    engine:
        description:
          - The database engine to return version details for.
        type: str
        choices:
          - aurora-mysql
          - aurora-postgresql
          - custom-oracle-ee
          - db2-ae
          - db2-se
          - mariadb
          - mysql
          - oracle-ee
          - oracle-ee-cdb
          - oracle-se2
          - oracle-se2-cdb
          - postgres
          - sqlserver-ee
          - sqlserver-se
          - sqlserver-ex
          - sqlserver-web
    engine_version:
        description:
          - A specific database engine version to return details for.
        type: str
    db_parameter_group_family:
        description:
          - The name of a specific RDS parameter group family to return details for.
        type: str
    default_only:
        description:
          - Specifies whether to return only the default version of the specified engine
            or the engine and major version combination.
        type: bool
        default: false
    filters:
        description:
            - A filter that specifies one or more DB engine versions to describe.
              See U(https://docs.aws.amazon.com/AmazonRDS/latest/APIReference/API_DescribeDBEngineVersions.html).
        type: dict
author:
  - Aubin Bikouo (@abikouo)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: List all of the available parameter group families for the Aurora PostgreSQL DB engine
  amazon.aws.rds_engine_versions_info:
    engine: aurora-postgresql

- name: List all of the available parameter group families for the Aurora PostgreSQL DB engine on a specific version
  amazon.aws.rds_engine_versions_info:
    engine: aurora-postgresql
    engine_version: 16.1

- name: Get default engine version for DB parameter group family postgres16
  amazon.aws.rds_engine_versions_info:
    engine: postgres
    default_only: true
    db_parameter_group_family: postgres16
"""

RETURN = r"""
db_engine_versions:
  description: List of RDS engine versions.
  returned: always
  type: list
  contains:
    engine:
        description:
        - The name of the database engine.
        type: str
    engine_version:
        description:
        - The version number of the database engine.
        type: str
    db_parameter_group_family:
        description:
        - The name of the DB parameter group family for the database engine.
        type: str
    db_engine_description:
        description:
        - The description of the database engine.
        type: str
    db_engine_version_description:
        description:
        - The description of the database engine version.
        type: str
    default_character_set:
        description:
        - The default character set for new instances of this engine version.
        type: dict
        sample: {
            "character_set_description": "Unicode 5.0 UTF-8 Universal character set",
            "character_set_name": "AL32UTF8"
        }
    image:
        description:
        - The EC2 image
        type: complex
        contains:
            image_id:
                description:
                - A value that indicates the ID of the AMI.
                type: str
            status:
                description:
                - A value that indicates the status of a custom engine version (CEV).
                type: str
    db_engine_media_type:
        description:
        - A value that indicates the source media provider of the AMI based on the usage operation.
        type: str
    supported_character_sets:
        description:
        - A list of the character sets supported by this engine for the CharacterSetName parameter of the CreateDBInstance operation.
        type: list
        elements: dict
        contains:
            character_set_name:
                description:
                - The name of the character set.
                type: str
            character_set_description:
                description:
                - The description of the character set.
                type: str
    supported_nchar_character_sets:
        description:
        - A list of the character sets supported by the Oracle DB engine.
        type: list
        elements: dict
        contains:
            character_set_name:
                description:
                - The name of the character set.
                type: str
            character_set_description:
                description:
                - The description of the character set.
                type: str
    valid_upgrade_target:
        description:
        - A list of engine versions that this database engine version can be upgraded to.
        type: list
        elements: dict
        sample: [
            {
                "auto_upgrade": false,
                "description": "Aurora PostgreSQL (Compatible with PostgreSQL 15.5)",
                "engine": "aurora-postgresql",
                "engine_version": "15.5",
                "is_major_version_upgrade": false,
                "supported_engine_modes": [
                    "provisioned"
                ],
                "supports_babelfish": true,
                "supports_global_databases": true,
                "supports_integrations": false,
                "supports_local_write_forwarding": true,
                "supports_parallel_query": false
            }
        ]
    supported_timezones:
        description:
        - A list of the time zones supported by this engine for the Timezone parameter of the CreateDBInstance action.
        type: list
        elements: dict
        sample: [
            {"TimezoneName": "xxx"}
        ]
    exportable_log_types:
        description:
        - The types of logs that the database engine has available for export to CloudWatch Logs.
        type: list
        elements: str
    supports_log_exports_to_cloudwatchLogs:
        description:
        - Indicates whether the engine version supports exporting the log types specified by ExportableLogTypes to CloudWatch Logs.
        type: bool
    supports_read_replica:
        description:
        - Indicates whether the database engine version supports read replicas.
        type: bool
    supported_engine_modes:
        description:
        - A list of the supported DB engine modes.
        type: list
        elements: str
    supported_feature_names:
        description:
        - A list of features supported by the DB engine.
        type: list
        elements: str
        sample: [
            "Comprehend",
            "Lambda",
            "s3Export",
            "s3Import",
            "SageMaker"
        ]
    status:
        description:
        - The status of the DB engine version, either available or deprecated.
        type: str
    supports_parallel_query:
        description:
        - Indicates whether you can use Aurora parallel query with a specific DB engine version.
        type: bool
    supports_global_databases:
        description:
        - Indicates whether you can use Aurora global databases with a specific DB engine version.
        type: bool
    major_engine_version:
        description:
        - The major engine version of the CEV.
        type: str
    database_installation_files_s3_bucket_name:
        description:
        - The name of the Amazon S3 bucket that contains your database installation files.
        type: str
    database_installation_files_s3_prefix:
        description:
        - The Amazon S3 directory that contains the database installation files.
        type: str
    db_engine_version_arn:
        description:
        - The ARN of the custom engine version.
        type: str
    kms_key_id:
        description:
        - The Amazon Web Services KMS key identifier for an encrypted CEV.
        type: str
    create_time:
        description:
        - The creation time of the DB engine version.
        type: str
    tags:
        description: A dictionary of key value pairs.
        type: dict
        sample: {
            "some": "tag"
        }
    supports_babelfish:
        description:
        - Indicates whether the engine version supports Babelfish for Aurora PostgreSQL.
        type: bool
    custom_db_engine_version_manifest:
        description:
        - JSON string that lists the installation files and parameters that RDS Custom uses to create a custom engine version (CEV).
        type: str
    supports_certificate_rotation_without_restart:
        description:
        - Indicates whether the engine version supports rotating the server certificate without rebooting the DB instance.
        type: bool
    supported_ca_certificate_identifiers:
        description:
        - A list of the supported CA certificate identifiers.
        type: list
        elements: str
        sample: [
            "rds-ca-2019",
            "rds-ca-ecc384-g1",
            "rds-ca-rsa4096-g1",
            "rds-ca-rsa2048-g1"
        ]
    supports_local_write_forwarding:
        description:
        - Indicates whether the DB engine version supports forwarding write operations from reader DB instances to the writer DB instance in the DB cluster.
        type: bool
    supports_integrations:
        description:
        - Indicates whether the DB engine version supports zero-ETL integrations with Amazon Redshift.
        type: bool
"""

from typing import Any
from typing import Dict
from typing import List

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


@AWSRetry.jittered_backoff(retries=10)
def _describe_db_engine_versions(connection: Any, **params: Dict[str, Any]) -> List[Dict[str, Any]]:
    paginator = connection.get_paginator("describe_db_engine_versions")
    return paginator.paginate(**params).build_full_result()["DBEngineVersions"]


def describe_db_engine_versions(connection: Any, module: AnsibleAWSModule) -> Dict[str, Any]:
    engine = module.params.get("engine")
    engine_version = module.params.get("engine_version")
    db_parameter_group_family = module.params.get("db_parameter_group_family")
    default_only = module.params.get("default_only")
    filters = module.params.get("filters")

    params = {"DefaultOnly": default_only}
    if engine:
        params["Engine"] = engine
    if engine_version:
        params["EngineVersion"] = engine_version
    if db_parameter_group_family:
        params["DBParameterGroupFamily"] = db_parameter_group_family
    if filters:
        params["Filters"] = filters

    try:
        result = _describe_db_engine_versions(connection, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, "Couldn't get RDS engine versions.")

    def _transform_item(v):
        tag_list = v.pop("TagList", [])
        v = camel_dict_to_snake_dict(v)
        v["tags"] = boto3_tag_list_to_ansible_dict(tag_list)
        return v

    return dict(changed=False, db_engine_versions=[_transform_item(v) for v in result])


def main() -> None:
    argument_spec = dict(
        engine=dict(
            choices=[
                "aurora-mysql",
                "aurora-postgresql",
                "custom-oracle-ee",
                "db2-ae",
                "db2-se",
                "mariadb",
                "mysql",
                "oracle-ee",
                "oracle-ee-cdb",
                "oracle-se2",
                "oracle-se2-cdb",
                "postgres",
                "sqlserver-ee",
                "sqlserver-se",
                "sqlserver-ex",
                "sqlserver-web",
            ]
        ),
        engine_version=dict(),
        db_parameter_group_family=dict(),
        default_only=dict(type="bool", default=False),
        filters=dict(type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        client = module.client("rds", retry_decorator=AWSRetry.jittered_backoff(retries=10))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    module.exit_json(**describe_db_engine_versions(client, module))


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import WorkflowJob


@pytest.mark.django_db
def test_bulk_job_launch(run_module, admin_user, job_template):
    jobs = [dict(unified_job_template=job_template.id)]
    result = run_module(
        'bulk_job_launch',
        {
            'name': "foo-bulk-job",
            'jobs': jobs,
            'extra_vars': {'animal': 'owl'},
            'limit': 'foo',
            'wait': False,
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    bulk_job = WorkflowJob.objects.get(name="foo-bulk-job")
    assert bulk_job.extra_vars == '{"animal": "owl"}'
    assert bulk_job.limit == "foo"


@pytest.mark.django_db
def test_bulk_host_create(run_module, admin_user, inventory):
    hosts = [dict(name="127.0.0.1"), dict(name="foo.dns.org")]
    result = run_module(
        'bulk_host_create',
        {
            'inventory': inventory.name,
            'hosts': hosts,
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result
    resp_hosts = inventory.hosts.all().values_list('name', flat=True)
    for h in hosts:
        assert h['name'] in resp_hosts


@pytest.mark.django_db
def test_bulk_host_delete(run_module, admin_user, inventory):
    hosts = [dict(name="127.0.0.1"), dict(name="foo.dns.org")]
    result = run_module(
        'bulk_host_create',
        {
            'inventory': inventory.name,
            'hosts': hosts,
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result
    resp_hosts_ids = list(inventory.hosts.all().values_list('id', flat=True))
    result = run_module(
        'bulk_host_delete',
        {
            'hosts': resp_hosts_ids,
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The arg spec for the eos_bgp_address_family module
"""


class Bgp_afArgs(object):  # pylint: disable=R0903
    """The arg spec for the eos_bgp_address_family module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "running_config": {"type": "str"},
        "state": {
            "default": "merged",
            "type": "str",
            "choices": [
                "deleted",
                "merged",
                "overridden",
                "replaced",
                "gathered",
                "rendered",
                "parsed",
            ],
        },
        "config": {
            "type": "dict",
            "options": {
                "as_number": {"type": "str"},
                "address_family": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "network": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "route_map": {"type": "str"},
                                "address": {"type": "str"},
                            },
                        },
                        "redistribute": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "ospf_route": {
                                    "type": "str",
                                    "choices": [
                                        "internal",
                                        "external",
                                        "nssa_external_1",
                                        "nssa_external_2",
                                    ],
                                },
                                "route_map": {"type": "str"},
                                "protocol": {
                                    "type": "str",
                                    "choices": ["isis", "ospfv3", "dhcp"],
                                },
                                "isis_level": {
                                    "type": "str",
                                    "choices": [
                                        "level-1",
                                        "level-2",
                                        "level-1-2",
                                    ],
                                },
                            },
                        },
                        "route_target": {
                            "type": "dict",
                            "options": {
                                "action": {
                                    "type": "str",
                                    "choices": ["both", "import", "export"],
                                    "aliases": ["mode"],
                                },
                                "type": {
                                    "type": "str",
                                    "choices": [
                                        "evpn",
                                        "vpn-ipv4",
                                        "vpn-ipv6",
                                    ],
                                },
                                "route_map": {"type": "str"},
                                "target": {"type": "str"},
                                "imported_route": {"type": "bool"},
                            },
                        },
                        "graceful_restart": {"type": "bool"},
                        "bgp_params": {
                            "type": "dict",
                            "options": {
                                "next_hop_address_family": {
                                    "type": "str",
                                    "choices": ["ipv6"],
                                },
                                "redistribute_internal": {"type": "bool"},
                                "route": {"type": "str"},
                                "additional_paths": {
                                    "type": "str",
                                    "choices": ["install", "send", "receive"],
                                },
                                "next_hop_unchanged": {"type": "bool"},
                            },
                        },
                        "safi": {
                            "type": "str",
                            "choices": ["labeled-unicast", "multicast"],
                        },
                        "neighbor": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "activate": {"type": "bool"},
                                "graceful_restart": {"type": "bool"},
                                "weight": {"type": "int"},
                                "default_originate": {
                                    "type": "dict",
                                    "options": {
                                        "route_map": {"type": "str"},
                                        "always": {"type": "bool"},
                                    },
                                },
                                "route_map": {
                                    "type": "dict",
                                    "options": {
                                        "direction": {
                                            "type": "str",
                                            "choices": ["in", "out"],
                                        },
                                        "name": {"type": "str"},
                                    },
                                },
                                "next_hop_address_family": {
                                    "type": "str",
                                    "choices": ["ipv6"],
                                },
                                "additional_paths": {
                                    "type": "str",
                                    "choices": ["send", "receive"],
                                },
                                "peer": {"type": "str"},
                                "encapsulation": {
                                    "type": "dict",
                                    "options": {
                                        "transport": {
                                            "type": "str",
                                            "choices": ["mpls", "vxlan"],
                                        },
                                        "source_interface": {"type": "str"},
                                    },
                                },
                                "prefix_list": {
                                    "type": "dict",
                                    "options": {
                                        "direction": {
                                            "type": "str",
                                            "choices": ["in", "out"],
                                        },
                                        "name": {"type": "str"},
                                    },
                                },
                                "next_hop_unchanged": {"type": "bool"},
                            },
                        },
                        "afi": {
                            "type": "str",
                            "choices": ["ipv4", "ipv6", "evpn"],
                        },
                        "vrf": {"type": "str"},
                    },
                },
            },
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: lan_automation_log_by_serial_number_info
short_description: Information module for Lan Automation Log By Serial Number Info
description:
- This module represents an alias of the module lan_automation_log_by_serial_number_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. LAN Automation session identifier.
    type: str
  serialNumber:
    description:
    - SerialNumber path parameter. Device serial number.
    type: str
  logLevel:
    description:
    - >
      LogLevel query parameter. Supported levels are ERROR, INFO, WARNING, TRACE, CONFIG and ALL. Specifying ALL
      will display device specific logs with the exception of CONFIG logs. In order to view CONFIG logs along with
      the remaining logs, please leave the query parameter blank.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for LAN Automation LANAutomationLogsForIndividualDevicesV1
  description: Complete reference of the LANAutomationLogsForIndividualDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-logs-for-individual-devices
notes:
  - SDK Method used are
    lan_automation.LanAutomation.lan_automation_logs_for_individual_devices_v1,

  - Paths used are
    get /dna/intent/api/v1/lan-automation/log/{id}/{serialNumber},
  - It should be noted that this module is an alias of lan_automation_log_by_serial_number_v1_info

"""

EXAMPLES = r"""
- name: Get Lan Automation Log By Serial Number Info by id
  cisco.dnac.lan_automation_log_by_serial_number_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    logLevel: string
    id: string
    serialNumber: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "nwOrchId": "string",
          "logs": [
            {
              "logLevel": "string",
              "timeStamp": "string",
              "record": "string"
            }
          ],
          "serialNumber": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2018 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualmachineextension_info
version_added: "0.1.2"
short_description: Get Azure Virtual Machine Extension facts
description:
    - Get facts of Azure Virtual Machine Extension.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    virtual_machine_name:
        description:
            - The name of the virtual machine containing the extension.
        required: True
        type: str
    name:
        description:
            - The name of the virtual machine extension.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Get information on specific Virtual Machine Extension
  azure_rm_virtualmachineextension_info:
    resource_group: myResourceGroup
    virtual_machine_name: myvm
    name: myextension

- name: List installed Virtual Machine Extensions
  azure_rm_virtualmachineextension_info:
    resource_group: myResourceGroup
    virtual_machine_name: myvm
'''

RETURN = '''
extensions:
    description:
        - A list of dictionaries containing facts for Virtual Machine Extension.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/myvm/testVM/extens
                     ions/myextension"
        resource_group:
            description:
                - Resource group name.
            returned: always
            type: str
            sample: myResourceGroup
        virtual_machine_name:
            description:
                - Virtual machine name.
            returned: always
            type: str
            sample: myvm
        name:
            description:
                - Virtual machine name.
            returned: always
            type: str
            sample: myextension
        location:
            description:
                - The resource location.
            returned: always
            type: str
            sample: eastus
        publisher:
            description:
                - Extension publisher.
            returned: always
            type: str
            sample: Microsoft.Azure.Extensions
        type:
            description:
                - Extension type.
            returned: always
            type: str
            sample: CustomScript
        settings:
            description:
                - Extension specific settings dictionary.
            returned: always
            type: dict
            sample: { 'commandToExecute':'hostname' }
        auto_upgrade_minor_version:
            description:
                - Auto upgrade minor version flag.
            returned: always
            type: bool
            sample: true
        enable_automatic_upgrade:
            description:
                - Enable automatic upgrade flag.
            returned: always
            type: bool
            sample: true
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { "mytag":"abc" }
        provisioning_state:
            description:
                - Provisioning state of the extension.
            returned: always
            type: str
            sample: Succeeded
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMVirtualMachineExtensionInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            virtual_machine_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.virtual_machine_name = None
        self.name = None
        self.tags = None
        super(AzureRMVirtualMachineExtensionInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):
        is_old_facts = self.module._name == 'azure_rm_virtualmachineextension_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_virtualmachineextension_facts' module has been renamed to 'azure_rm_virtualmachineextension_info'",
                                  version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['extensions'] = self.get_extensions()
        else:
            self.results['extensions'] = self.list_extensions()

        return self.results

    def get_extensions(self):
        response = None
        results = []
        try:
            response = self.compute_client.virtual_machine_extensions.get(resource_group_name=self.resource_group,
                                                                          vm_name=self.virtual_machine_name,
                                                                          vm_extension_name=self.name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Virtual Machine Extension.')

        if response and self.has_tags(response.tags, self.tags):
            results.append(self.format_response(response))

        return results

    def list_extensions(self):
        response = None
        results = []
        try:
            response = self.compute_client.virtual_machine_extensions.list(resource_group_name=self.resource_group,
                                                                           vm_name=self.virtual_machine_name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Virtual Machine Extension.')

        if response is not None and response.value is not None:
            for item in response.value:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_response(item))

        return results

    def format_response(self, item):
        d = item.as_dict()
        d = {
            'id': d.get('id', None),
            'resource_group': self.resource_group,
            'virtual_machine_name': self.virtual_machine_name,
            'location': d.get('location'),
            'name': d.get('name'),
            'publisher': d.get('publisher'),
            'type': d.get('type_properties_type'),
            'settings': d.get('settings'),
            'auto_upgrade_minor_version': d.get('auto_upgrade_minor_version'),
            'enable_automatic_upgrade': d.get('enable_automatic_upgrade', None),
            'tags': d.get('tags', None),
            'provisioning_state': d.get('provisioning_state')
        }
        return d


def main():
    AzureRMVirtualMachineExtensionInfo()


if __name__ == '__main__':
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import InstanceGroup, Instance


@pytest.mark.django_db
def test_instance_group_create(run_module, admin_user):
    result = run_module(
        'instance_group', {'name': 'foo-group', 'policy_instance_percentage': 34, 'policy_instance_minimum': 12, 'state': 'present'}, admin_user
    )
    assert not result.get('failed', False), result
    assert result['changed']

    ig = InstanceGroup.objects.get(name='foo-group')
    assert ig.policy_instance_percentage == 34
    assert ig.policy_instance_minimum == 12

    # Create a new instance in the DB
    new_instance = Instance.objects.create(hostname='foo.example.com')

    # Set the new instance group only to the one instnace
    result = run_module('instance_group', {'name': 'foo-group', 'instances': [new_instance.hostname], 'state': 'present'}, admin_user)
    assert not result.get('failed', False), result
    assert result['changed']

    ig = InstanceGroup.objects.get(name='foo-group')
    all_instance_names = []
    for instance in ig.instances.all():
        all_instance_names.append(instance.hostname)

    assert new_instance.hostname in all_instance_names, 'Failed to add instance to group'
    assert len(all_instance_names) == 1, 'Too many instances in group {0}'.format(','.join(all_instance_names))


@pytest.mark.django_db
def test_container_group_create(run_module, admin_user, kube_credential):
    pod_spec = "{ 'Nothing': True }"

    result = run_module('instance_group', {'name': 'foo-c-group', 'credential': kube_credential.id, 'is_container_group': True, 'state': 'present'}, admin_user)
    assert not result.get('failed', False), result['msg']
    assert result['changed']

    ig = InstanceGroup.objects.get(name='foo-c-group')
    assert ig.pod_spec_override == ''

    result = run_module(
        'instance_group',
        {'name': 'foo-c-group', 'credential': kube_credential.id, 'is_container_group': True, 'pod_spec_override': pod_spec, 'state': 'present'},
        admin_user,
    )
    assert not result.get('failed', False), result['msg']
    assert result['changed']

    ig = InstanceGroup.objects.get(name='foo-c-group')
    assert ig.pod_spec_override == pod_spec



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: health_score_definitions_v1_info
short_description: Information module for Health Score Definitions V1
description:
- Get all Health Score Definitions V1.
- Get Health Score Definitions V1 by id.
- Get all health score defintions.
- >
   Get health score defintion for the given id. Definition includes all properties from HealthScoreDefinition schema
   by default. For detailed information about the usage of the API, please refer to the Open API specification
   document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-issueAndHealthDefinitions-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceType:
    description:
    - >
      DeviceType query parameter. These are the device families supported for health score definitions. If no
      input is made on device family, all device families are considered.
    type: str
  id:
    description:
    - >
      Id query parameter. The definition identifier. Examples id=015d9cba-4f53-4087-8317-7e49e5ffef46 (single
      entity id request) id=015d9cba-4f53-4087-8317-7e49e5ffef46&id=015d9cba-4f53-4087-8317-7e49e5ffef47 (multiple
      ids in the query param).
    type: str
  includeForOverallHealth:
    description:
    - >
      IncludeForOverallHealth query parameter. The inclusion status of the issue definition, either true or false.
      True indicates that particular health metric is included in overall health computation, otherwise false. By
      default it's set to true.
    type: bool
  attribute:
    description:
    - >
      Attribute query parameter. These are the attributes supported in health score definitions response. By
      default, all properties are sent in response.
    type: str
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetAllHealthScoreDefinitionsForGivenFiltersV1
  description: Complete reference of the GetAllHealthScoreDefinitionsForGivenFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-all-health-score-definitions-for-given-filters
- name: Cisco DNA Center documentation for Devices GetHealthScoreDefinitionForTheGivenIdV1
  description: Complete reference of the GetHealthScoreDefinitionForTheGivenIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-health-score-definition-for-the-given-id
notes:
  - SDK Method used are
    devices.Devices.get_all_health_score_definitions_for_given_filters_v1,
    devices.Devices.get_health_score_definition_for_the_given_id_v1,

  - Paths used are
    get /dna/intent/api/v1/healthScoreDefinitions,
    get /dna/intent/api/v1/healthScoreDefinitions/{id},

"""

EXAMPLES = r"""
- name: Get all Health Score Definitions V1
  cisco.dnac.health_score_definitions_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceType: string
    id: string
    includeForOverallHealth: True
    attribute: string
    offset: 0
    limit: 0
  register: result

- name: Get Health Score Definitions V1 by id
  cisco.dnac.health_score_definitions_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "name": "string",
          "displayName": "string",
          "deviceFamily": "string",
          "description": "string",
          "includeForOverallHealth": true,
          "definitionStatus": "string",
          "thresholdValue": 0,
          "synchronizeToIssueThreshold": true,
          "lastModified": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr logging_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.logging_global.logging_global import (
    Logging_globalArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.logging_global import (
    Logging_globalTemplate,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import (
    flatten_config,
)


class Logging_globalFacts(object):
    """The iosxr logging_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Logging_globalArgs.argument_spec

    def get_config(self, connection):
        return connection.get("show running-config logging")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Logging_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_config(connection)

        flatten_context_list = [
            "logging archive",
            "logging tls-server",
            "logging correlator rule",
            "logging correlator ruleset",
            "logging events filter",
            "logging buffered discriminator",
            "logging monitor discriminator",
            "logging console discriminator",
        ]

        for x in flatten_context_list:
            data = flatten_config(data, x)
        # parse native config using the Logging_global template
        logging_global_parser = Logging_globalTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = logging_global_parser.parse()
        objs["tls_servers"] = list(objs.get("tls_servers", {}).values())
        if objs.get("correlator"):
            objs["correlator"]["rules"] = list(
                objs.get("correlator", {}).get("rules", {}).values(),
            )
            objs["correlator"]["rule_sets"] = list(
                objs.get("correlator", {}).get("rule_sets", {}).values(),
            )
            for i, x in enumerate(objs["correlator"]["rule_sets"]):
                if None in x["rulename"]:
                    objs["correlator"]["rule_sets"][i]["rulename"].remove(None)

        ansible_facts["ansible_network_resources"].pop("logging_global", None)

        params = utils.remove_empties(
            logging_global_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["logging_global"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_virtual_network_ip_pool
short_description: Resource module for Sda Virtual Network Ip Pool
description:
- This module represents an alias of the module sda_virtual_network_ip_pool_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  autoGenerateVlanName:
    description: It will auto generate vlanName, if vlanName is empty(applicable for
      L3 and INFRA_VN).
    type: bool
  ipPoolName:
    description: IpPoolName query parameter.
    type: str
    version_added: 4.0.0
  isBridgeModeVm:
    description: Bridge Mode Vm enablement flag (applicable for L3 and L2 and default
      value is False ).
    type: bool
  isCommonPool:
    description: Common Pool enablement flag(applicable for L3 and L2 and default value
      is False ).
    type: bool
  isIpDirectedBroadcast:
    description: Ip Directed Broadcast enablement flag(applicable for L3 and default
      value is False ).
    type: bool
  isL2FloodingEnabled:
    description: Layer2 flooding enablement flag(applicable for L3 , L2 and always true
      for L2 and default value is False ).
    type: bool
    version_added: 4.0.0
  isLayer2Only:
    description: Layer2 Only enablement flag and default value is False.
    type: bool
  isThisCriticalPool:
    description: Critical pool enablement flag(applicable for L3 and default value is
      False ).
    type: bool
    version_added: 4.0.0
  isWirelessPool:
    description: Wireless Pool enablement flag(applicable for L3 and L2 and default
      value is False ).
    type: bool
    version_added: 4.0.0
  poolType:
    description: Pool Type (applicable for INFRA_VN).
    type: str
    version_added: 4.0.0
  scalableGroupName:
    description: Scalable Group Name(applicable for L3).
    type: str
    version_added: 4.0.0
  siteNameHierarchy:
    description: SiteNameHierarchy query parameter.
    type: str
    version_added: 4.0.0
  trafficType:
    description: Traffic type(applicable for L3 and L2).
    type: str
    version_added: 4.0.0
  virtualNetworkName:
    description: VirtualNetworkName query parameter.
    type: str
  vlanId:
    description: Vlan Id(applicable for L3 , L2 and INFRA_VN).
    type: str
  vlanName:
    description: Vlan name represent the segment name, if empty, vlanName would be auto
      generated by API.
    type: str
    version_added: 4.0.0
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddIPPoolInSDAVirtualNetworkV1
  description: Complete reference of the AddIPPoolInSDAVirtualNetworkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-ip-pool-in-sda-virtual-network
- name: Cisco DNA Center documentation for SDA DeleteIPPoolFromSDAVirtualNetworkV1
  description: Complete reference of the DeleteIPPoolFromSDAVirtualNetworkV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-ip-pool-from-sda-virtual-network
notes:
  - SDK Method used are
    sda.Sda.add_ip_pool_in_sda_virtual_network_v1,
    sda.Sda.delete_ip_pool_from_sda_virtual_network_v1,

  - Paths used are
    post /dna/intent/api/v1/business/sda/virtualnetwork/ippool,
    delete /dna/intent/api/v1/business/sda/virtualnetwork/ippool,
  - It should be noted that this module is an alias of sda_virtual_network_ip_pool_v1

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.sda_virtual_network_ip_pool:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    ipPoolName: string
    siteNameHierarchy: string
    virtualNetworkName: string

- name: Create
  cisco.dnac.sda_virtual_network_ip_pool:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    autoGenerateVlanName: true
    ipPoolName: string
    isBridgeModeVm: true
    isCommonPool: true
    isIpDirectedBroadcast: true
    isL2FloodingEnabled: true
    isLayer2Only: true
    isThisCriticalPool: true
    isWirelessPool: true
    poolType: string
    scalableGroupName: string
    siteNameHierarchy: string
    trafficType: string
    virtualNetworkName: string
    vlanId: string
    vlanName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: discovery_job_info
short_description: Information module for Discovery Job Info
description:
- This module represents an alias of the module discovery_job_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  offset:
    description:
    - Offset query parameter.
    type: int
  limit:
    description:
    - Limit query parameter.
    type: int
  ipAddress:
    description:
    - IpAddress query parameter.
    type: str
  name:
    description:
    - Name query parameter.
    type: str
  id:
    description:
    - Id path parameter. Discovery ID.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery GetDiscoveryJobsByIPV1
  description: Complete reference of the GetDiscoveryJobsByIPV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-discovery-jobs-by-ip
- name: Cisco DNA Center documentation for Discovery GetListOfDiscoveriesByDiscoveryIdV1
  description: Complete reference of the GetListOfDiscoveriesByDiscoveryIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-list-of-discoveries-by-discovery-id
notes:
  - SDK Method used are
    discovery.Discovery.get_discovery_jobs_by_ip_v1,
    discovery.Discovery.get_list_of_discoveries_by_discovery_id_v1,

  - Paths used are
    get /dna/intent/api/v1/discovery/job,
    get /dna/intent/api/v1/discovery/{id}/job,
  - It should be noted that this module is an alias of discovery_job_v1_info

"""

EXAMPLES = r"""
- name: Get all Discovery Job Info
  cisco.dnac.discovery_job_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    ipAddress: string
    name: string
  register: result

- name: Get Discovery Job Info by id
  cisco.dnac.discovery_job_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    offset: 0
    limit: 0
    ipAddress: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "attributeInfo": {},
          "cliStatus": "string",
          "discoveryStatus": "string",
          "endTime": "string",
          "httpStatus": "string",
          "id": "string",
          "inventoryCollectionStatus": "string",
          "inventoryReachabilityStatus": "string",
          "ipAddress": "string",
          "jobStatus": "string",
          "name": "string",
          "netconfStatus": "string",
          "pingStatus": "string",
          "snmpStatus": "string",
          "startTime": "string",
          "taskId": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_dhcp_option_info
version_added: 1.0.0
short_description: Gather information about DHCP options sets in AWS
description:
  - Gather information about DHCP options sets in AWS.
author: "Nick Aslanidis (@naslanidis)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeDhcpOptions.html) for possible filters.
    type: dict
    default: {}
  dhcp_options_ids:
    description:
      - Get details of specific DHCP option IDs.
    type: list
    elements: str
  dry_run:
    description:
      - Checks whether you have the required permissions to view the DHCP
        options.
    type: bool
    default: false
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# # Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all DHCP Option sets for an account or profile
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
  register: dhcp_info

- name: Gather information about a filtered list of DHCP Option sets
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
    filters:
      "tag:Name": "abc-123"
  register: dhcp_info

- name: Gather information about a specific DHCP Option set by DhcpOptionId
  amazon.aws.ec2_vpc_dhcp_option_info:
    region: ap-southeast-2
    profile: production
    dhcp_options_ids: dopt-123fece2
  register: dhcp_info
"""

RETURN = r"""
dhcp_options:
    description: The DHCP options created, associated or found.
    returned: always
    type: list
    elements: dict
    contains:
        dhcp_configurations:
            description: The DHCP configuration for the option set.
            type: list
            elements: dict
            contains:
                key:
                    description: The name of a DHCP option.
                    returned: always
                    type: str
                values:
                    description: List of values for the DHCP option.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        value:
                            description: The attribute value. This value is case-sensitive.
                            returned: always
                            type: str
            sample:
              - '{"key": "ntp-servers", "values": [{"value": "10.0.0.2" , "value": "10.0.1.2"}]}'
              - '{"key": "netbios-name-servers", "values": [{value": "10.0.0.1"}, {"value": "10.0.1.1" }]}'
        dhcp_options_id:
            description: The aws resource id of the primary DHCP options set created or found.
            type: str
            sample: "dopt-0955331de6a20dd07"
        owner_id:
            description: The ID of the AWS account that owns the DHCP options set.
            type: str
            sample: 012345678912
        tags:
            description: The tags to be applied to a DHCP options set.
            type: list
            elements: dict
            sample:
              - '{"Key": "CreatedBy", "Value": "ansible-test"}'
              - '{"Key": "Collection", "Value": "amazon.aws"}'
dhcp_config:
    description: The boto2-style DHCP options created, associated or found. Provided for consistency with ec2_vpc_dhcp_option's C(dhcp_config).
    returned: always
    type: list
    elements: dict
    contains:
      domain-name-servers:
        description: The IP addresses of up to four domain name servers, or AmazonProvidedDNS.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      domain-name:
        description: The domain name for hosts in the DHCP option sets.
        returned: when available
        type: list
        sample:
          - "my.example.com"
      ntp-servers:
        description: The IP addresses of up to four Network Time Protocol (NTP) servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-name-servers:
        description: The IP addresses of up to four NetBIOS name servers.
        returned: when available
        type: list
        sample:
          - 10.0.0.1
          - 10.0.1.1
      netbios-node-type:
        description: The NetBIOS node type (1, 2, 4, or 8).
        returned: when available
        type: str
        sample: 2
changed:
    description: True if listing the dhcp options succeeds.
    type: bool
    returned: always
"""

from typing import Any
from typing import Dict
from typing import List
from typing import Tuple

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_dhcp_options
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import normalize_ec2_vpc_dhcp_config
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def list_dhcp_options(client, module: AnsibleAWSModule) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:
    params = dict(Filters=ansible_dict_to_boto3_filter_list(module.params.get("filters")))

    if module.params.get("dry_run"):
        params["DryRun"] = True

    if module.params.get("dhcp_options_ids"):
        params["DhcpOptionsIds"] = module.params.get("dhcp_options_ids")

    all_dhcp_options = describe_dhcp_options(client, **params)
    if not all_dhcp_options and module.params.get("dhcp_options_ids"):
        module.fail_json(msg="Requested DHCP options does not exist")

    normalized_config = [normalize_ec2_vpc_dhcp_config(config["DhcpConfigurations"]) for config in all_dhcp_options]
    raw_config = [
        camel_dict_to_snake_dict(
            {
                "DhcpOptionsId": option["DhcpOptionsId"],
                "DhcpConfigurations": option["DhcpConfigurations"],
                "Tags": boto3_tag_list_to_ansible_dict(option.get("Tags", [{"Value": "", "Key": "Name"}])),
            },
            ignore_list=["Tags"],
        )
        for option in all_dhcp_options
    ]
    return raw_config, normalized_config


def main() -> None:
    argument_spec = dict(
        filters=dict(type="dict", default={}),
        dry_run=dict(type="bool", default=False),
        dhcp_options_ids=dict(type="list", elements="str"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    client = module.client("ec2")

    # call your function here
    try:
        results, normalized_config = list_dhcp_options(client, module)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    module.exit_json(dhcp_options=results, dhcp_config=normalized_config)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr_route_maps config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.route_maps import (
    Route_mapsTemplate,
)


class Route_maps(ResourceModule):
    """
    The iosxr_route_maps config class
    """

    def __init__(self, module):
        super(Route_maps, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="route_maps",
            tmplt=Route_mapsTemplate(),
        )
        self.parsers = [
            "add.eigrp_metric",
            "add.rip_metric",
            "drop",
            "pass",
            "prepend",
            "suppress_route",
            "unsuppress_route",
            "remove",
            "set.administrative_distance",
            "set.aigp_metric",
            "set.attribute_set",
            "set.c_multicast_routing",
            "set.community",
            "set.core_tree",
            "set.dampening",
            "set.downstream_core_tree",
            "set.eigrp_metric",
            "set.fallback_vrf_lookup",
            "set.flow_tag",
            "set.forward_class",
            "set.ip_precedence",
            "set.isis_metric",
            "set.label",
            "set.label_index",
            "set.label_mode",
            "set.large_community",
            "set.level",
            "set.load_balance",
            "set.lsm_root",
            "set.metric_type",
            "set.mpls",
            "set.next_hop",
            "set.origin",
            "set.ospf_metric",
            "set.path_selection.all",
            "set.path_selection.backup",
            "set.path_selection.best_path",
            "set.path_selection.group_best",
            "set.path_selection.multiplath",
            "set.path_color",
            "set.qos_group",
            "set.rib_metric",
            "set.rip_metric",
            "set.rip_tag",
            "set.rt_set",
            "set.s_pmsi",
            "set.spf_priority",
            "set.static_p2mp_te",
            "set.tag",
            "set.traffic_index",
            "set.upstream_core_tree",
            "set.vpn_distinguisher",
            "set.weight",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self._route_maps_list_to_dict(self.want)
        haved = self._route_maps_list_to_dict(self.have)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        for k, want in iteritems(wantd):
            if self.state == "purged":  # for purged state
                if haved.pop(k, {}):
                    self._handle_purged(k)
            else:  # for all other states
                self._compare(want=want, have=haved.pop(k, {}), policy_name=k)

        # clean anything that is surplus, if state purged clean all have if want is empty
        if self.state == "overridden" or (self.state == "purged" and not wantd):
            for h, haved in iteritems(haved):
                self._handle_purged(h)

    def _handle_purged(self, policy_name):
        self.commands.append(f"no route-policy {policy_name}")

    def _compare(self, want, have, policy_name):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Route_maps network resource.
        """
        append_endif = False
        append_nested_endif = False
        append_else_once = True
        order_list = [
            "global",
            "if_section_",
            "elseif_section_",
            "elseHas_global_",
            "elseHas_if_section_",
            "elseHas_elseif_section_",
            "elseHas_else_section_",
        ]  # to maintain the sanity of how commands are generated
        begin = len(self.commands)

        for check_cond in order_list:  # iterate on the list to preserve sequence
            w_res = {key: val for key, val in want.items() if key.startswith(check_cond)}

            for w_condition, w_policy_config in w_res.items():  # loop over want's condition section
                h_policy_config = have.pop(w_condition, {})

                # if want clauses and have clauses are not same
                if w_policy_config != h_policy_config:
                    if self.state in ["replaced", "overridden"]:
                        # cannot add commands on a adhoc manner it replaces the whole config
                        h_policy_config = {}

                    render_condition = {
                        "condition": w_policy_config.pop("condition", ""),
                        "condition_type": w_policy_config.pop("conf_type").split("_section")[0],
                    }  # required to generate conditional statements

                    begin_endif = len(self.commands)  # handle elseif conditions

                    if check_cond.startswith("elseHas_"):
                        # adds else only once if there is else block
                        if append_else_once:
                            self.commands.append("else")
                            append_else_once = False

                    if render_condition.get("condition_type") != "global":
                        self.addcmd(
                            render_condition,
                            "condition",
                            negate=False,
                        )  # condition commands added here
                    if w_policy_config.get("apply"):  # as apply is a list
                        w_apply_config = w_policy_config.pop("apply", {})
                        h_apply_config = h_policy_config.pop("apply", {})
                        for w_name, w_apply in w_apply_config.items():
                            h_apply = h_apply_config.pop(w_name, {})
                            # apply config added here
                            self.compare(
                                parsers=[
                                    "apply",
                                ],
                                want={"apply": w_apply},
                                have={"apply": h_apply},
                            )
                    # route-policy configs added here
                    self.compare(parsers=self.parsers, want=w_policy_config, have=h_policy_config)
                    if len(self.commands) != begin_endif and w_condition.startswith("if_section_"):
                        # if we want to add any condition we have to start with if
                        append_endif = True
                    if len(self.commands) != begin_endif and w_condition.startswith(
                        "elseHas_if_section_",
                    ):
                        append_nested_endif = True  # same as above

        if len(self.commands) != begin:
            if append_nested_endif:  # add endif if there was a nested else
                self.commands.append("endif")
            if append_endif:  # add endif if there was a condition in the top level config
                self.commands.append("endif")
            self.commands.append("end-policy")  # if route-policy then end-policy
            self.commands.insert(
                begin,
                f"route-policy {policy_name}",
            )  # the name of the route-policy

    def _route_maps_list_to_dict(self, data):
        temp_rmap_list = dict()

        def process_apply(apply_conf):
            rm_apply = {}
            for apply_config in apply_conf:
                rm_apply[apply_config.get("route_policy")] = apply_config
            return rm_apply

        for rmap in data:
            temp_rmap = dict()
            rmap_name = ""
            for cond, rm_conf in rmap.items():
                if cond == "name":
                    rmap_name = rm_conf
                    temp_rmap["name"] = rmap_name
                elif cond in ["if_section", "global"]:
                    if rm_conf.get("apply"):
                        rm_conf["apply"] = process_apply(rm_conf.get("apply"))
                    rm_conf["conf_type"] = cond
                    if cond == "global":
                        temp_rmap[cond] = rm_conf
                    else:
                        temp_rmap[cond + "_" + (rm_conf.get("condition").replace(" ", "_"))] = (
                            rm_conf
                        )
                elif cond == "elseif_section":
                    for elif_config in rm_conf:
                        if elif_config.get("apply"):
                            elif_config["apply"] = process_apply(elif_config.get("apply"))
                        elif_config["conf_type"] = cond
                        temp_rmap[cond + "_" + (elif_config.get("condition").replace(" ", "_"))] = (
                            elif_config
                        )
                elif (
                    cond == "else_section"
                ):  # wanted to do recursion but the overall performance is better this way
                    for else_cond, else_rm_conf in rm_conf.items():
                        if else_cond in ["if_section", "global", "else_section"]:
                            if else_rm_conf.get("apply"):
                                else_rm_conf["apply"] = process_apply(else_rm_conf.get("apply"))
                            else_rm_conf["conf_type"] = else_cond
                            if else_cond in ["global", "else_section"]:
                                temp_rmap["elseHas_" + else_cond + "_"] = else_rm_conf
                            else:
                                temp_rmap[
                                    "elseHas_"
                                    + else_cond
                                    + "_"
                                    + (else_rm_conf.get("condition").replace(" ", "_"))
                                ] = else_rm_conf
                        elif else_cond == "elseif_section":
                            for elif_config in else_rm_conf:
                                if elif_config.get("apply"):
                                    elif_config["apply"] = process_apply(elif_config.get("apply"))
                                elif_config["conf_type"] = "elseif_section"
                                temp_rmap[
                                    "elseHas_"
                                    + else_cond
                                    + "_"
                                    + (elif_config.get("condition").replace(" ", "_"))
                                ] = elif_config
            if temp_rmap:
                temp_rmap_list[rmap_name] = temp_rmap
        return temp_rmap_list



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_wireless_settings_ssids_v1
short_description: Resource module for Sites Wireless Settings Ssids V1
description:
- Manage operations create, update and delete of the resource Sites Wireless Settings Ssids V1.
- This API allows the user to create an SSID Service Set Identifier at the Global site.
- >
   This API allows the user to delete an SSID Service Set Identifier at the global level, if the SSID is not mapped
   to any Wireless Profile.
- This API allows the user to update an SSID Service Set Identifier at the given site.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aaaOverride:
    description: Activate the AAA Override feature when set to true.
    type: bool
  acctServers:
    description: List of Accounting server IpAddresses.
    elements: str
    type: list
  aclName:
    description: Pre-Auth Access Control List (ACL) Name.
    type: str
  authServer:
    description: Authentication Server, Mandatory for Guest SSIDs with wlanType=Guest
      and l3AuthType=web_auth.
    type: str
  authServers:
    description: List of Authentication/Authorization server IpAddresses.
    elements: str
    type: list
  authType:
    description: L2 Authentication Type (If authType is not open , then atleast one
      RSN Cipher Suite and corresponding valid AKM must be enabled).
    type: str
  basicServiceSetClientIdleTimeout:
    description: This refers to the duration of inactivity, measured in seconds, before
      a client connected to the Basic Service Set is considered idle and timed out.
    type: int
  basicServiceSetMaxIdleEnable:
    description: Activate the maximum idle feature for the Basic Service Set.
    type: bool
  cckmTsfTolerance:
    description: Cckm TImestamp Tolerance(in milliseconds).
    type: int
  clientExclusionEnable:
    description: Activate the feature that allows for the exclusion of clients.
    type: bool
  clientExclusionTimeout:
    description: This refers to the length of time, in seconds, a client is excluded
      or blocked from accessing the network after a specified number of unsuccessful
      attempts.
    type: int
  clientRateLimit:
    description: This pertains to the maximum data transfer rate, specified in bits
      per second, that a client is permitted to achieve.
    type: int
  coverageHoleDetectionEnable:
    description: Activate Coverage Hole Detection feature when set to true.
    type: bool
  directedMulticastServiceEnable:
    description: The Directed Multicast Service feature becomes operational when it
      is set to true.
    type: bool
  egressQos:
    description: Egress QOS.
    type: str
  externalAuthIpAddress:
    description: External WebAuth URL (Mandatory for Guest SSIDs with wlanType = Guest,
      l3AuthType = web_auth and authServer = auth_external).
    type: str
  fastTransition:
    description: Fast Transition.
    type: str
  fastTransitionOverTheDistributedSystemEnable:
    description: Enable Fast Transition over the Distributed System when set to true.
    type: bool
  ghz24Policy:
    description: 2.4 Ghz Band Policy value. Allowed only when 2.4 Radio Band is enabled
      in ssidRadioType.
    type: str
  ghz6PolicyClientSteering:
    description: True if 6 GHz Policy Client Steering is enabled, else False.
    type: bool
  id:
    description: Id path parameter. SSID ID. Inputs containing special characters should
      be encoded.
    type: str
  ingressQos:
    description: Ingress QOS.
    type: str
  isApBeaconProtectionEnabled:
    description: When set to true, the Access Point (AP) Beacon Protection feature is
      activated, enhancing the security of the network.
    type: bool
  isAuthKey8021x:
    description: When set to true, the 802.1X authentication key is in use.
    type: bool
  isAuthKey8021xPlusFT:
    description: When set to true, the 802.1X-Plus-FT authentication key is in use.
    type: bool
  isAuthKey8021x_SHA256:
    description: When set to true, the feature that enables 802.1X authentication using
      the SHA256 algorithm is turned on.
    type: bool
  isAuthKeyEasyPSK:
    description: When set to true, the feature that enables the use of Easy Pre-shared
      Key (PSK) authentication is activated.
    type: bool
  isAuthKeyOWE:
    description: When set to true, the Opportunistic Wireless Encryption (OWE) authentication
      key feature is turned on.
    type: bool
  isAuthKeyPSK:
    description: When set to true, the Pre-shared Key (PSK) authentication feature is
      enabled.
    type: bool
  isAuthKeyPSKPlusFT:
    description: When set to true, the feature that enables the combination of Pre-shared
      Key (PSK) and Fast Transition (FT) authentication keys is activated.
    type: bool
  isAuthKeyPSKSHA256:
    description: The feature that allows the use of Pre-shared Key (PSK) authentication
      with the SHA256 algorithm is enabled when it is set to true.
    type: bool
  isAuthKeySae:
    description: When set to true, the feature enabling the Simultaneous Authentication
      of Equals (SAE) authentication key is activated.
    type: bool
  isAuthKeySaeExt:
    description: When set to true, the Simultaneous Authentication of Equals (SAE) Extended
      Authentication key feature is turned on.
    type: bool
  isAuthKeySaeExtPlusFT:
    description: When set to true, the Simultaneous Authentication of Equals (SAE) combined
      with Fast Transition (FT) Authentication Key feature is enabled.
    type: bool
  isAuthKeySaePlusFT:
    description: Activating this setting by switching it to true turns on the authentication
      key feature that supports both Simultaneous Authentication of Equals (SAE) and
      Fast Transition (FT).
    type: bool
  isAuthKeySuiteB1921x:
    description: When set to true, the SuiteB192-1x authentication key feature is enabled.
    type: bool
  isAuthKeySuiteB1x:
    description: When activated by setting it to true, the SuiteB-1x authentication
      key feature is engaged.
    type: bool
  isBroadcastSSID:
    description: When activated by setting it to true, the Broadcast SSID feature will
      make the SSID publicly visible to wireless devices searching for available networks.
    type: bool
  isCckmEnabled:
    description: True if CCKM is enabled, else False.
    type: bool
  isEnabled:
    description: Set SSID's admin status as 'Enabled' when set to true.
    type: bool
  isFastLaneEnabled:
    description: True if FastLane is enabled, else False.
    type: bool
  isHex:
    description: True if passphrase is in Hex format, else False.
    type: bool
  isMacFilteringEnabled:
    description: When set to true, MAC Filtering will be activated, allowing control
      over network access based on the MAC address of the device.
    type: bool
  isPosturingEnabled:
    description: Applicable only for Enterprise SSIDs. When set to True, Posturing will
      enabled. Required to be set to True if ACL needs to be mapped for Enterprise SSID.
    type: bool
  isRandomMacFilterEnabled:
    description: Deny clients using randomized MAC addresses when set to true.
    type: bool
  l3AuthType:
    description: L3 Authentication Type.
    type: str
  managementFrameProtectionClientprotection:
    description: Management Frame Protection Client.
    type: str
  multiPSKSettings:
    description: Sites Wireless Settings Ssids's multiPSKSettings.
    elements: dict
    suboptions:
      passphrase:
        description: Passphrase needs to be between 8 and 63 characters for ASCII type.
          HEX passphrase needs to be 64 characters.
        type: str
      passphraseType:
        description: Passphrase Type.
        type: str
      priority:
        description: Priority.
        type: int
    type: list
  nasOptions:
    description: Pre-Defined NAS Options AP ETH Mac Address, AP IP address, AP Location
      , AP MAC Address, AP Name, AP Policy Tag, AP Site Tag, SSID, System IP Address,
      System MAC Address, System Name.
    elements: str
    type: list
  neighborListEnable:
    description: The Neighbor List feature is enabled when it is set to true.
    type: bool
  openSsid:
    description: Open SSID which is already created in the design and not associated
      to any other OPEN-SECURED SSID.
    type: str
  passphrase:
    description: Passphrase (Only applicable for SSID with PERSONAL security level).
      Passphrase needs to be between 8 and 63 characters for ASCII type. HEX passphrase
      needs to be 64 characters.
    type: str
  profileName:
    description: WLAN Profile Name, if not passed autogenerated profile name will be
      assigned. The same wlanProfileName will also be used for policyProfileName.
    type: str
  protectedManagementFrame:
    description: (REQUIRED is applicable for authType WPA3_PERSONAL, WPA3_ENTERPRISE,
      OPEN_SECURED) and (OPTIONAL/REQUIRED is applicable for authType WPA2_WPA3_PERSONAL
      and WPA2_WPA3_ENTERPRISE).
    type: str
  rsnCipherSuiteCcmp128:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite CCMP128
      encryption protocol is activated.
    type: bool
  rsnCipherSuiteCcmp256:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite CCMP256
      encryption protocol is activated.
    type: bool
  rsnCipherSuiteGcmp128:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite GCMP128
      encryption protocol is activated.
    type: bool
  rsnCipherSuiteGcmp256:
    description: When set to true, the Robust Security Network (RSN) Cipher Suite GCMP256
      encryption protocol is activated.
    type: bool
  sessionTimeOut:
    description: This denotes the allotted time span, expressed in seconds, before a
      session is automatically terminated due to inactivity.
    type: int
  sessionTimeOutEnable:
    description: Turn on the feature that imposes a time limit on user sessions.
    type: bool
  siteId:
    description: SiteId path parameter. Site UUID of Global site.
    type: str
  sleepingClientEnable:
    description: When set to true, this will activate the timeout settings that apply
      to clients in sleep mode.
    type: bool
  sleepingClientTimeout:
    description: This refers to the amount of time, measured in minutes, before a sleeping
      (inactive) client is timed out of the network.
    type: int
  ssid:
    description: Name of the SSID.
    type: str
  ssidRadioType:
    description: Radio Policy Enum (default Triple band operation(2.4GHz, 5GHz and 6GHz)).
    type: str
  webPassthrough:
    description: When set to true, the Web-Passthrough feature will be activated for
      the Guest SSID, allowing guests to bypass certain login requirements.
    type: bool
  wlanBandSelectEnable:
    description: Band select is allowed only when band options selected contains at
      least 2.4 GHz and 5 GHz band.
    type: bool
  wlanType:
    description: Wlan Type.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateSSIDV1
  description: Complete reference of the CreateSSIDV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-ssid
- name: Cisco DNA Center documentation for Wireless DeleteSSIDV1
  description: Complete reference of the DeleteSSIDV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-ssid
- name: Cisco DNA Center documentation for Wireless UpdateSSIDV1
  description: Complete reference of the UpdateSSIDV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-ssid
notes:
  - SDK Method used are
    wireless.Wireless.create_ssid_v1,
    wireless.Wireless.delete_ssid_v1,
    wireless.Wireless.update_ssid_v1,

  - Paths used are
    post /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids,
    delete /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids/{id},
    put /dna/intent/api/v1/sites/{siteId}/wirelessSettings/ssids/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sites_wireless_settings_ssids_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    aaaOverride: true
    acctServers:
    - string
    aclName: string
    authServer: string
    authServers:
    - string
    authType: string
    basicServiceSetClientIdleTimeout: 0
    basicServiceSetMaxIdleEnable: true
    cckmTsfTolerance: 0
    clientExclusionEnable: true
    clientExclusionTimeout: 0
    clientRateLimit: 0
    coverageHoleDetectionEnable: true
    directedMulticastServiceEnable: true
    egressQos: string
    externalAuthIpAddress: string
    fastTransition: string
    fastTransitionOverTheDistributedSystemEnable: true
    ghz24Policy: string
    ghz6PolicyClientSteering: true
    ingressQos: string
    isApBeaconProtectionEnabled: true
    isAuthKey8021x: true
    isAuthKey8021xPlusFT: true
    isAuthKey8021x_SHA256: true
    isAuthKeyEasyPSK: true
    isAuthKeyOWE: true
    isAuthKeyPSK: true
    isAuthKeyPSKPlusFT: true
    isAuthKeyPSKSHA256: true
    isAuthKeySae: true
    isAuthKeySaeExt: true
    isAuthKeySaeExtPlusFT: true
    isAuthKeySaePlusFT: true
    isAuthKeySuiteB1921x: true
    isAuthKeySuiteB1x: true
    isBroadcastSSID: true
    isCckmEnabled: true
    isEnabled: true
    isFastLaneEnabled: true
    isHex: true
    isMacFilteringEnabled: true
    isPosturingEnabled: true
    isRandomMacFilterEnabled: true
    l3AuthType: string
    managementFrameProtectionClientprotection: string
    multiPSKSettings:
    - passphrase: string
      passphraseType: string
      priority: 0
    nasOptions:
    - string
    neighborListEnable: true
    openSsid: string
    passphrase: string
    profileName: string
    protectedManagementFrame: string
    rsnCipherSuiteCcmp128: true
    rsnCipherSuiteCcmp256: true
    rsnCipherSuiteGcmp128: true
    rsnCipherSuiteGcmp256: true
    sessionTimeOut: 0
    sessionTimeOutEnable: true
    siteId: string
    sleepingClientEnable: true
    sleepingClientTimeout: 0
    ssid: string
    ssidRadioType: string
    webPassthrough: true
    wlanBandSelectEnable: true
    wlanType: string

- name: Update by id
  cisco.dnac.sites_wireless_settings_ssids_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    aaaOverride: true
    acctServers:
    - string
    aclName: string
    authServer: string
    authServers:
    - string
    authType: string
    basicServiceSetClientIdleTimeout: 0
    basicServiceSetMaxIdleEnable: true
    cckmTsfTolerance: 0
    clientExclusionEnable: true
    clientExclusionTimeout: 0
    clientRateLimit: 0
    coverageHoleDetectionEnable: true
    directedMulticastServiceEnable: true
    egressQos: string
    externalAuthIpAddress: string
    fastTransition: string
    fastTransitionOverTheDistributedSystemEnable: true
    ghz24Policy: string
    ghz6PolicyClientSteering: true
    id: string
    ingressQos: string
    isApBeaconProtectionEnabled: true
    isAuthKey8021x: true
    isAuthKey8021xPlusFT: true
    isAuthKey8021x_SHA256: true
    isAuthKeyEasyPSK: true
    isAuthKeyOWE: true
    isAuthKeyPSK: true
    isAuthKeyPSKPlusFT: true
    isAuthKeyPSKSHA256: true
    isAuthKeySae: true
    isAuthKeySaeExt: true
    isAuthKeySaeExtPlusFT: true
    isAuthKeySaePlusFT: true
    isAuthKeySuiteB1921x: true
    isAuthKeySuiteB1x: true
    isBroadcastSSID: true
    isCckmEnabled: true
    isEnabled: true
    isFastLaneEnabled: true
    isHex: true
    isMacFilteringEnabled: true
    isPosturingEnabled: true
    isRandomMacFilterEnabled: true
    l3AuthType: string
    managementFrameProtectionClientprotection: string
    multiPSKSettings:
    - passphrase: string
      passphraseType: string
      priority: 0
    nasOptions:
    - string
    neighborListEnable: true
    openSsid: string
    passphrase: string
    profileName: string
    protectedManagementFrame: string
    rsnCipherSuiteCcmp128: true
    rsnCipherSuiteCcmp256: true
    rsnCipherSuiteGcmp128: true
    rsnCipherSuiteGcmp256: true
    sessionTimeOut: 0
    sessionTimeOutEnable: true
    siteId: string
    sleepingClientEnable: true
    sleepingClientTimeout: 0
    ssid: string
    ssidRadioType: string
    webPassthrough: true
    wlanBandSelectEnable: true
    wlanType: string

- name: Delete by id
  cisco.dnac.sites_wireless_settings_ssids_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string
    siteId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-


# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: label
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller labels.
description:
    - Create, update, or destroy Automation Platform Controller labels. See
      U(https://www.ansible.com/tower) for an overview.
    - Note, labels can only be created via the API, they can not be deleted.
      Once they are fully disassociated the API will clean them up on its own.
options:
    name:
      description:
        - Name of this label.
      required: True
      type: str
    new_name:
      description:
        - Setting this option will change the existing name (looked up via the name field).
      type: str
    organization:
      description:
        - Organization name, ID, or named URL this label belongs to.
      required: True
      type: str
    state:
      description:
        - Desired state of the resource.
      default: "present"
      choices: ["present", "exists"]
      type: str
extends_documentation_fragment: awx.awx.auth
'''

EXAMPLES = '''
- name: Add label to organization
  label:
    name: Custom Label
    organization: My Organization
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        organization=dict(required=True),
        state=dict(choices=['present', 'exists'], default='present'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get("new_name")
    organization = module.params.get('organization')
    state = module.params.get("state")

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    organization_id = None
    if organization:
        organization_id = module.resolve_name_to_id('organizations', organization)

    # Attempt to look up an existing item based on the provided data
    existing_item = module.get_one(
        'labels',
        name_or_id=name,
        check_exists=(state == 'exists'),
        **{
            'data': {
                'organization': organization_id,
            }
        }
    )

    # Create the data that gets sent for create and update
    new_fields = {}
    new_fields['name'] = new_name if new_name else (module.get_item_name(existing_item) if existing_item else name)
    if organization:
        new_fields['organization'] = organization_id

    module.create_or_update_if_needed(existing_item, new_fields, endpoint='labels', item_type='label', associations={})


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: golden_tag_image_details_v1_info
short_description: Information module for Golden Tag Image Details V1
description:
- Get Golden Tag Image Details V1 by id.
- Get golden tag status of an image. Set siteId as -1 for Global site.
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - SiteId path parameter. Site Id in uuid format. Set siteId as -1 for Global site.
    type: str
  deviceFamilyIdentifier:
    description:
    - DeviceFamilyIdentifier path parameter. Device family identifier e.g. 277696480-283933147, e.g. 277696480.
    type: str
  deviceRole:
    description:
    - >
      DeviceRole path parameter. Device Role. Permissible Values ALL, UNKNOWN, ACCESS, BORDER ROUTER, DISTRIBUTION
      and CORE.
    type: str
  imageId:
    description:
    - ImageId path parameter. Image Id in uuid format.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) GetGoldenTagStatusOfAnImageV1
  description: Complete reference of the GetGoldenTagStatusOfAnImageV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-golden-tag-status-of-an-image
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.get_golden_tag_status_of_an_image_v1,

  - Paths used are
    get /dna/intent/api/v1/image/importation/golden/site/{siteId}/family/{deviceFamilyIdentifier}/role/{deviceRole}/image/{imageId},

"""

EXAMPLES = r"""
- name: Get Golden Tag Image Details V1 by id
  cisco.dnac.golden_tag_image_details_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    deviceFamilyIdentifier: string
    deviceRole: string
    imageId: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "deviceRole": "string",
        "taggedGolden": true,
        "inheritedSiteName": "string",
        "inheritedSiteId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: assurance_issues_query
short_description: Resource module for Assurance Issues Query
description:
- This module represents an alias of the module assurance_issues_query_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  endTime:
    description: End Time.
    type: int
  filters:
    description: Assurance Issues Query's filters.
    elements: dict
    suboptions:
      filters:
        description: Assurance Issues Query's filters.
        elements: dict
        suboptions:
          key:
            description: Key.
            type: str
          operator:
            description: Operator.
            type: str
          value:
            description: Value.
            type: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description: Start Time.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTheDetailsOfIssuesForGivenSetOfFiltersV1
  description: Complete reference of the GetTheDetailsOfIssuesForGivenSetOfFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-details-of-issues-for-given-set-of-filters
notes:
  - SDK Method used are
    issues.Issues.get_the_details_of_issues_for_given_set_of_filters_v1,

  - Paths used are
    post /dna/data/api/v1/assuranceIssues/query,
  - It should be noted that this module is an alias of assurance_issues_query_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assurance_issues_query:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    endTime: 0
    filters:
    - filters:
      - key: string
        operator: string
        value: string
      key: string
      logicalOperator: string
      operator: string
      value: string
    headers: '{{my_headers | from_json}}'
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "issueId": "string",
          "name": "string",
          "description": "string",
          "summary": "string",
          "priority": "string",
          "severity": "string",
          "deviceType": "string",
          "category": "string",
          "entityType": "string",
          "entityId": "string",
          "firstOccurredTime": 0,
          "mostRecentOccurredTime": 0,
          "status": "string",
          "isGlobal": true,
          "updatedBy": {},
          "updatedTime": {},
          "notes": {},
          "siteId": {},
          "siteHierarchyId": {},
          "siteName": {},
          "siteHierarchy": {},
          "suggestedActions": [
            {
              "message": "string",
              "steps": [
                {}
              ]
            }
          ],
          "additionalAttributes": [
            {
              "key": "string",
              "value": "string"
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0,
        "sortBy": [
          {
            "name": "string",
            "order": "string"
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_user_right
short_description: Manage Windows User Rights
description:
- Add, remove or set User Rights for a group or users or groups.
- You can set user rights for both local and domain accounts.
options:
  name:
    description:
    - The name of the User Right as shown by the C(Constant Name) value from
      U(https://learn.microsoft.com/en-us/windows/security/threat-protection/security-policy-settings/user-rights-assignment).
    - The module will return an error if the right is invalid.
    type: str
    required: yes
  users:
    description:
    - A list of users or groups to add/remove on the User Right.
    - These can be in the form DOMAIN\user-group, user-group@DOMAIN.COM for
      domain users/groups.
    - For local users/groups it can be in the form user-group, .\user-group,
      SERVERNAME\user-group where SERVERNAME is the name of the remote server.
    - It is highly recommended to use the C(.\) or C(SERVERNAME\) prefix to
      avoid any ambiguity with domain account names or errors trying to lookup
      an account on a domain controller.
    - You can also add special local accounts like SYSTEM and others.
    - Can be set to an empty list with I(action=set) to remove all accounts
      from the right.
    type: list
    elements: str
    required: yes
  action:
    description:
    - C(add) will add the users/groups to the existing right.
    - C(remove) will remove the users/groups from the existing right.
    - C(set) will replace the users/groups of the existing right.
    type: str
    default: set
    choices: [ add, remove, set ]
notes:
- If the server is domain joined this module can change a right but if a GPO
  governs this right then the changes won't last.
seealso:
- module: ansible.windows.win_group
- module: ansible.windows.win_group_membership
- module: ansible.windows.win_user
author:
- Jordan Borean (@jborean93)
'''

EXAMPLES = r'''
---
- name: Replace the entries of Deny log on locally
  ansible.windows.win_user_right:
    name: SeDenyInteractiveLogonRight
    users:
      - Guest
      - Users
    action: set

- name: Add account to Log on as a service
  ansible.windows.win_user_right:
    name: SeServiceLogonRight
    users:
      - .\Administrator
      - '{{ansible_hostname}}\local-user'
    action: add

- name: Remove accounts who can create Symbolic links
  ansible.windows.win_user_right:
    name: SeCreateSymbolicLinkPrivilege
    users:
      - SYSTEM
      - Administrators
      - DOMAIN\User
      - group@DOMAIN.COM
    action: remove

- name: Remove all accounts who cannot log on remote interactively
  ansible.windows.win_user_right:
    name: SeDenyRemoteInteractiveLogonRight
    users: []
'''

RETURN = r'''
added:
  description: A list of accounts that were added to the right, this is empty
    if no accounts were added.
  returned: success
  type: list
  sample: ["NT AUTHORITY\\SYSTEM", "DOMAIN\\User"]
removed:
  description: A list of accounts that were removed from the right, this is
    empty if no accounts were removed.
  returned: success
  type: list
  sample: ["SERVERNAME\\Administrator", "BUILTIN\\Administrators"]
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_issues_top_n_analytics_v1
short_description: Resource module for Assurance Issues Top N Analytics V1
description:
- Manage operation create of the resource Assurance Issues Top N Analytics V1.
- >
   Gets the Top N analytics data related to issues based on given filters and group by field. This data can be used
   to find top sites which has most issues or top device types with most issue etc,. Https //github.com/cisco-en-
   programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-IssuesList-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Assurance Issues Top N Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Assurance Issues Top N Analytics's filters.
    elements: dict
    suboptions:
      filters:
        description: Assurance Issues Top N Analytics's filters.
        elements: dict
        suboptions:
          key:
            description: Key.
            type: str
          operator:
            description: Operator.
            type: str
          value:
            description: Value.
            type: str
        type: list
      key:
        description: Key.
        type: str
      logicalOperator:
        description: Logical Operator.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Assurance Issues Top N Analytics's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Assurance Issues Top N Analytics's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  topN:
    description: Top N.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTopNAnalyticsDataOfIssuesV1
  description: Complete reference of the GetTopNAnalyticsDataOfIssuesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-top-n-analytics-data-of-issues
notes:
  - SDK Method used are
    issues.Issues.get_top_n_analytics_data_of_issues_v1,

  - Paths used are
    post /dna/data/api/v1/assuranceIssues/topNAnalytics,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assurance_issues_top_n_analytics_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - filters:
      - key: string
        operator: string
        value: string
      key: string
      logicalOperator: string
      operator: string
      value: string
    groupBy:
    - string
    headers: '{{my_headers | from_json}}'
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    topN: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": [
        {
          "id": "string",
          "attributes": [
            {
              "name": "string",
              "value": "string"
            }
          ],
          "aggregateAttributes": [
            {
              "name": "string",
              "function": "string",
              "value": 0
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0,
        "sortBy": [
          {
            "name": "string",
            "function": {},
            "order": "string"
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: upgrade_stage_cancel
short_description: Resource module for Upgrade Stage Cancel
description:
- Manage operation create of the resource Upgrade Stage Cancel.
- API to cancel staging process of specified nodes from PPAN.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  hostnames:
    description: Upgrade Stage Cancel's hostnames.
    elements: str
    type: list
  preCheckReportID:
    description: Upgrade Stage Cancel's preCheckReportID.
    type: str
  upgradeType:
    description: Upgrade Stage Cancel's upgradeType.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    fullupgrade.Fullupgrade.cancel_staging_on_p_p_a_n,

  - Paths used are
    post /api/v1/upgrade/stage/cancel-stage,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.upgrade_stage_cancel:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    hostnames:
    - string
    preCheckReportID: string
    upgradeType: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "dbStatus": "string",
        "message": "string",
        "node": "string",
        "percentage": 0,
        "progressMsg": "string",
        "status": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: images_v1_info
short_description: Information module for Images V1
description:
- Get all Images V1.
- >
   A list of available images for the specified site is provided. The default value of the site is set to global. The
   list includes images that have been imported onto the disk, as well as the latest and suggested images from
   Cisco.com.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - >
      SiteId query parameter. Site identifier to get the list of all available products under the site. The
      default value is the global site. See https //developer.cisco.com/docs/dna-center/get-site for `siteId`.
    type: str
  productNameOrdinal:
    description:
    - >
      ProductNameOrdinal query parameter. The product name ordinal is a unique value for each network device
      product. The productNameOrdinal can be obtained from the response of API
      `/dna/intent/api/v1/siteWiseProductNames`.
    type: float
  supervisorProductNameOrdinal:
    description:
    - >
      SupervisorProductNameOrdinal query parameter. The supervisor engine module ordinal is a unique value for
      each supervisor module. The `supervisorProductNameOrdinal` can be obtained from the response of API
      `/dna/intent/api/v1/siteWiseProductNames`.
    type: float
  imported:
    description:
    - >
      Imported query parameter. When the value is set to `true`, it will include physically imported images.
      Conversely, when the value is set to `false`, it will include image records from the cloud. The identifier
      for cloud images can be utilized to download images from Cisco.com to the disk.
    type: bool
  name:
    description:
    - >
      Name query parameter. Filter with software image or add-on name. Supports partial case-insensitive search. A
      minimum of 3 characters is required for the search.
    type: str
  version:
    description:
    - >
      Version query parameter. Filter with image version. Supports partial case-insensitive search. A minimum of 3
      characters is required for the search.
    type: str
  golden:
    description:
    - >
      Golden query parameter. When set to `true`, it will retrieve the images marked as tagged golden. When set to
      `false`, it will retrieve the images marked as not tagged golden.
    type: bool
  integrity:
    description:
    - >
      Integrity query parameter. Filter with verified images using Integrity Verification Available values
      UNKNOWN, VERIFIED.
    type: str
  hasAddonImages:
    description:
    - >
      HasAddonImages query parameter. When set to `true`, it will retrieve the images which have add-on images.
      When set to `false`, it will retrieve the images which do not have add-on images.
    type: bool
  isAddonImages:
    description:
    - >
      IsAddonImages query parameter. When set to `true`, it will retrieve the images that an add-on image. When
      set to `false`, it will retrieve the images that are not add-on images.
    type: bool
  offset:
    description:
    - >
      Offset query parameter. The first record to show for this page; the first record is numbered 1. The minimum
      value is 1.
    type: float
  limit:
    description:
    - >
      Limit query parameter. The number of records to show for this page. The minimum and maximum values are 1 and
      500, respectively.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) ReturnsListOfSoftwareImagesV1
  description: Complete reference of the ReturnsListOfSoftwareImagesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!returns-list-of-software-images
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.returns_list_of_software_images_v1,

  - Paths used are
    get /dna/intent/api/v1/images,

"""

EXAMPLES = r"""
- name: Get all Images V1
  cisco.dnac.images_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    productNameOrdinal: 0
    supervisorProductNameOrdinal: 0
    imported: True
    name: string
    version: string
    golden: True
    integrity: string
    hasAddonImages: True
    isAddonImages: True
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "imported": true,
          "name": "string",
          "version": "string",
          "imageType": "string",
          "recommended": "string",
          "ciscoLatest": true,
          "integrityStatus": "string",
          "isAddonImage": true,
          "hasAddonImages": true,
          "goldenTaggingDetails": [
            {
              "deviceRoles": [
                "string"
              ],
              "deviceTags": [
                "string"
              ],
              "inheritedSiteId": "string",
              "inheritedSiteName": "string"
            }
          ],
          "productNames": [
            {
              "id": "string",
              "productName": "string",
              "productNameOrdinal": 0,
              "supervisorProductName": "string",
              "supervisorProductNameOrdinal": 0
            }
          ],
          "isGoldenTagged": true
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pnp_virtual_account_sync_result_v1_info
short_description: Information module for Pnp Virtual Account Sync Result V1
description:
- Get all Pnp Virtual Account Sync Result V1.
- Returns the summary of devices synced from the given smart account & virtual account with PnP Deprecated .
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  domain:
    description:
    - Domain path parameter. Smart Account Domain.
    type: str
  name:
    description:
    - Name path parameter. Virtual Account Name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) GetSyncResultForVirtualAccountV1
  description: Complete reference of the GetSyncResultForVirtualAccountV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-sync-result-for-virtual-account
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.get_sync_result_for_virtual_account_v1,

  - Paths used are
    get /dna/intent/api/v1/onboarding/pnp-device/sacct/{domain}/vacct/{name}/sync-result,

"""

EXAMPLES = r"""
- name: Get all Pnp Virtual Account Sync Result V1
  cisco.dnac.pnp_virtual_account_sync_result_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    domain: string
    name: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "virtualAccountId": "string",
      "autoSyncPeriod": 0,
      "syncResultStr": "string",
      "profile": {
        "proxy": true,
        "makeDefault": true,
        "port": 0,
        "profileId": "string",
        "name": "string",
        "addressIpV4": "string",
        "cert": "string",
        "addressFqdn": "string"
      },
      "ccoUser": "string",
      "syncResult": {
        "syncList": [
          {
            "syncType": "string",
            "deviceSnList": [
              "string"
            ]
          }
        ],
        "syncMsg": "string"
      },
      "token": "string",
      "syncStartTime": 0,
      "lastSync": 0,
      "tenantId": "string",
      "smartAccountId": "string",
      "expiry": 0,
      "syncStatus": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_transit_networks_v1
short_description: Resource module for Sda Transit Networks V1
description:
- Manage operations create, update and delete of the resource Sda Transit Networks V1.
- Adds transit networks based on user input.
- Updates transit networks based on user input.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Sda Transit Networks's payload.
    elements: dict
    suboptions:
      id:
        description: ID of the transit network (updating this field is not allowed).
        type: str
      ipTransitSettings:
        description: Sda Transit Networks's ipTransitSettings.
        suboptions:
          autonomousSystemNumber:
            description: Autonomous System Number of the IP transit network. Allowed
              range is 1 to 4294967295 (updating this field is not allowed).
            type: str
          routingProtocolName:
            description: Routing Protocol Name of the IP transit network (updating this
              field is not allowed).
            type: str
        type: dict
      name:
        description: Name of the transit network (updating this field is not allowed).
        type: str
      sdaTransitSettings:
        description: Sda Transit Networks's sdaTransitSettings.
        suboptions:
          controlPlaneNetworkDeviceIds:
            description: List of network device IDs that will be used as control plane
              nodes. Maximum 2 network device IDs can be provided for transit of type
              SDA_LISP_BGP_TRANSIT and maximum 4 network device IDs can be provided
              for transit of type SDA_LISP_PUB_SUB_TRANSIT.
            elements: str
            type: list
          isMulticastOverTransitEnabled:
            description: Set this to true to enable multicast over SD-Access transit.
              This supports Native Multicast over SD-Access Transit. This is only applicable
              for transit of type SDA_LISP_PUB_SUB_TRANSIT.
            type: bool
        type: dict
      type:
        description: Type of the transit network (updating this field is not allowed).
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddTransitNetworksV1
  description: Complete reference of the AddTransitNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-transit-networks
- name: Cisco DNA Center documentation for SDA UpdateTransitNetworksV1
  description: Complete reference of the UpdateTransitNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-transit-networks
notes:
  - SDK Method used are
    sda.Sda.add_transit_networks_v1,
    sda.Sda.update_transit_networks_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/transitNetworks,
    put /dna/intent/api/v1/sda/transitNetworks,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sda_transit_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - id: string
      ipTransitSettings:
        autonomousSystemNumber: string
        routingProtocolName: string
      name: string
      sdaTransitSettings:
        controlPlaneNetworkDeviceIds:
        - string
        isMulticastOverTransitEnabled: true
      type: string

- name: Create
  cisco.dnac.sda_transit_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - ipTransitSettings:
        autonomousSystemNumber: string
        routingProtocolName: string
      name: string
      sdaTransitSettings:
        controlPlaneNetworkDeviceIds:
        - string
        isMulticastOverTransitEnabled: true
      type: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The L2_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class L2_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(L2_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""^interface
                    (\s(?P<name>\S+))
                    $""",
                re.VERBOSE,
            ),
            "compval": "name",
            "setval": "interface {{ name }}",
            "result": {"{{ name }}": {"name": "{{ name }}"}},
            "shared": True,
        },
        {
            "name": "access.vlan",
            "getval": re.compile(
                r"""
                \s+switchport\saccess\svlan\s(?P<vlan>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "switchport access vlan {{ access.vlan }}",
            "remval": "switchport access vlan",
            "result": {
                "{{ name }}": {
                    "access": {
                        "vlan": "{{ vlan }}",
                    },
                },
            },
        },
        {
            "name": "access.vlan_name",
            "getval": re.compile(
                r"""
                \s+switchport\saccess\svlan\sname\s(?P<vlan_name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "switchport access vlan name {{ access.vlan_name }}",
            "result": {
                "{{ name }}": {
                    "access": {
                        "vlan_name": "{{ vlan_name }}",
                    },
                },
            },
        },
        {
            "name": "voice.vlan",
            "getval": re.compile(
                r"""
                \s+switchport\svoice\svlan\s(?P<vlan>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "switchport voice vlan {{ voice.vlan|string }}",
            "remval": "switchport voice vlan",
            "result": {
                "{{ name }}": {
                    "voice": {
                        "vlan": "{{ vlan }}",
                    },
                },
            },
        },
        {
            "name": "voice.vlan_tag",
            "getval": re.compile(
                r"""
                \s+switchport\svoice\svlan\s(?P<vlan_tag>dot1p|none|untagged)
                $""", re.VERBOSE,
            ),
            "setval": "switchport voice vlan {{ voice.vlan_tag }}",
            "remval": "switchport voice vlan",
            "result": {
                "{{ name }}": {
                    "voice": {
                        "vlan_tag": "{{ vlan_tag }}",
                    },
                },
            },
        },
        {
            "name": "voice.vlan_name",
            "getval": re.compile(
                r"""
                \s+switchport\svoice\svlan\sname\s(?P<vlan_name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "switchport voice vlan name {{ voice.vlan_name }}",
            "result": {
                "{{ name }}": {
                    "voice": {
                        "vlan_name": "{{ vlan_name }}",
                    },
                },
            },
        },
        {
            "name": "mode",
            "getval": re.compile(
                r"""
                \s+switchport\smode\s(?P<mode>access|trunk|dynamic|dynamic\sauto|dynamic\sdesirable|private-vlan\shost|private-vlan\spromiscuous|private-vlan\strunk\ssecondary|dot1q-tunnel)
                $""", re.VERBOSE,
            ),
            "setval": "switchport mode "
                      "{{ 'trunk' if mode == 'trunk' }}"
                      "{{ 'access' if mode == 'access' }}"
                      "{{ 'dynamic' if mode == 'dynamic' }}"
                      "{{ 'dynamic desirable' if mode == 'dynamic_desirable' }}"
                      "{{ 'dynamic auto' if mode == 'dynamic_auto' }}"
                      "{{ 'dot1q-tunnel' if mode == 'dot1q_tunnel' }}"
                      "{{ 'private-vlan host' if mode == 'private_vlan_host' }}"
                      "{{ 'private-vlan promiscuous' if mode == 'private_vlan_promiscuous' }}"
                      "{{ 'private-vlan trunk secondary' if mode == 'private_vlan_trunk' }}",
            "remval": "switchport mode",
            "result": {
                "{{ name }}": {
                    "mode": "{{ mode }}",
                },
            },
        },
        {
            "name": "trunk.allowed_vlans",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\sallowed\svlan\s(?P<allowed_vlan>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "allowed_vlans": "{{ allowed_vlan.split(',') }}",
                    },
                },
            },
        },
        {
            "name": "trunk.allowed_vlans_add",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\sallowed\svlan\sadd\s(?P<allowed_vlans_add>\S+$)
                """, re.VERBOSE,
            ),
            "setval": "",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "allowed_vlans_add": ["{{ allowed_vlans_add.split(',') }}"],
                    },
                },
            },
        },
        {
            "name": "trunk.encapsulation",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\sencapsulation\s(?P<encapsulation>dot1q|isl|negotiate)
                $""", re.VERBOSE,
            ),
            "setval": "switchport trunk encapsulation {{ trunk.encapsulation }}",
            "remval": "switchport trunk encapsulation",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "encapsulation": "{{ encapsulation }}",
                    },
                },
            },
        },
        {
            "name": "trunk.native_vlan",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\snative\svlan\s(?P<native_vlan>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "switchport trunk native vlan {{ trunk.native_vlan }}",
            "remval": "switchport trunk native vlan",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "native_vlan": "{{ native_vlan }}",
                    },
                },
            },
        },
        {
            "name": "trunk.pruning_vlans",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\spruning\svlan\s(?P<pruning_vlans>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "pruning_vlans": "{{ pruning_vlans.split(',') }}",
                    },
                },
            },
        },
        {
            "name": "trunk.pruning_vlans_add",
            "getval": re.compile(
                r"""
                \s+switchport\strunk\spruning\svlan\sadd\s(?P<pruning_vlans_add>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "",
            "result": {
                "{{ name }}": {
                    "trunk": {
                        "pruning_vlans_add": ["{{ pruning_vlans_add.split(',') }}"],
                    },
                },
            },
        },
    ]
    # fmt: on
